local AceGUI = LibStub("AceGUI-3.0")

----------------
-- Simple Main Frame --
----------------
--[[
    Events :
        OnClose

]]
do
   local Type = "SimpleFrame"
   local Version = 1
   
   local FrameBackdrop = {
      bgFile="Interface/Tooltips/UI-Tooltip-Background",
      edgeFile="Interface/Tooltips/UI-Tooltip-Border", 
      tile = true, tileSize = 16, edgeSize = 16, 
      insets = { left = 4, right = 4, top = 4, bottom = 4 }
   }
   
   local function frameOnClose(this)
      this.obj:Fire("OnClose")
   end
   
   local function closeOnClick(this)
      this.obj:Hide()
   end
   
   local function frameOnMouseDown(this)
		this:StartMoving()
      AceGUI:ClearFocus()
   end
   
   local function titleOnMouseDown(this)
      this:GetParent():StartMoving()
      AceGUI:ClearFocus()
   end
   
   local function frameOnMouseUp(this)
      local frame = this
      frame:StopMovingOrSizing()
      local self = frame.obj
      local status = self.status or self.localstatus
      status.width = frame:GetWidth()
      status.height = frame:GetHeight()
      status.top = frame:GetTop()
      status.left = frame:GetLeft()
   end
   
   local function Hide(self)
      self.frame:Hide()
   end
   
   local function Show(self)
      self.frame:Show()
   end
   
   local function OnAcquire(self)
      self.frame:SetParent(UIParent)
      self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
      self:ApplyStatus()
   end
   
   local function OnRelease(self)
      self.status = nil
      for k in pairs(self.localstatus) do
         self.localstatus[k] = nil
      end
   end
   
   -- called to set an external table to store status in
   local function SetStatusTable(self, status)
      assert(type(status) == "table")
      self.status = status
      self:ApplyStatus()
   end
   
   local function ApplyStatus(self)
      local status = self.status or self.localstatus
      local frame = self.frame
      self:SetWidth(status.width or 100)
      self:SetHeight(status.height or 100)
      if status.top and status.left then
         frame:SetPoint("TOP",UIParent,"BOTTOM",0,status.top)
         frame:SetPoint("LEFT",UIParent,"LEFT",status.left,0)
      else
         frame:SetPoint("CENTER",UIParent,"CENTER")
      end
   end
   
   local function OnWidthSet(self, width)
      local content = self.content
      local contentwidth = width - 34
      if contentwidth < 0 then
         contentwidth = 0
      end
      content:SetWidth(contentwidth)
      content.width = contentwidth
   end
   
   
   local function OnHeightSet(self, height)
      local content = self.content
      local contentheight = height - 34
      if contentheight < 0 then
         contentheight = 0
      end
      content:SetHeight(contentheight)
      content.height = contentheight
   end
   
   local function Constructor()
      local frame = CreateFrame("Frame",nil,UIParent)
      local self = {}
      self.type = Type
      
      self.Hide = Hide
      self.Show = Show
      self.OnRelease = OnRelease
      self.OnAcquire = OnAcquire
      self.SetStatusTable = SetStatusTable
      self.ApplyStatus = ApplyStatus
      self.OnWidthSet = OnWidthSet
      self.OnHeightSet = OnHeightSet
      
      self.localstatus = {}
      
      self.frame = frame
      frame.obj = self

      frame:SetWidth(100)
      frame:SetHeight(100)
      frame:SetPoint("CENTER",UIParent,"CENTER",0,0)
      frame:EnableMouse()
      frame:SetMovable(true)
      frame:SetFrameStrata("FULLSCREEN_DIALOG")
      frame:SetScript("OnMouseDown", frameOnMouseDown)
      frame:SetScript("OnMouseUp", frameOnMouseUp)
      
      frame:SetBackdrop(FrameBackdrop)
      frame:SetBackdropColor(0,0,0,1)
      frame:SetScript("OnHide",frameOnClose)
      frame:SetToplevel(true)
      
      --Container Support
      local content = CreateFrame("Frame",nil,frame)
      self.content = content
      content.obj = self
      content:SetPoint("TOPLEFT",frame,"TOPLEFT",17,-17)
      content:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",-17,17)
      
      AceGUI:RegisterAsContainer(self)
      return self    
   end
   
   AceGUI:RegisterWidgetType(Type,Constructor,Version)
end


